'''
Created on 12 mai 2021

@author: laborie
'''
from django.test import TestCase
from model_bakery import baker
from django.urls import reverse
from mri_links.models import LiensOutils
from mri_users.models import User
from django.test.utils import override_settings
import logging

def prevent_request_warnings(original_function):
    """
    If we need to test for 404s or 405s this decorator can prevent the
    request class from throwing warnings.
    """
    def new_function(*args, **kwargs):
        # raise logging level to ERROR
        logger = logging.getLogger('django.request')
        previous_logging_level = logger.getEffectiveLevel()
        logger.setLevel(logging.ERROR)

        # trigger original function that would throw warning
        original_function(*args, **kwargs)

        # lower logging level back to previous
        logger.setLevel(previous_logging_level)

    return new_function

@override_settings(AUTHENTICATION_BACKENDS=
                   ('django.contrib.auth.backends.ModelBackend',))
class ListeLiensViewTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        baker.make(LiensOutils, 10)

    def setUp(self):
        user1 = User.objects.create_user(username='testuser1',
                                         password='test1passw0rd')
        user2 = User.objects.create_user(username='testuser2',
                                         password='test2passw0rd')
        user1.save()
        user2.account_type = 'administrator'
        user2.save()

    def test_redirect_if_not_logged_in(self):
        response = self.client.get(reverse('manage_liens_outils'))
        self.assertRedirects(response, '/?next=/manage_liens_outils/')

    @prevent_request_warnings
    def test_forbidden_if_not_logged_with_admin_user(self):
        login = self.client.login(username='testuser1',
                                  password='test1passw0rd')
        self.assertTrue(login)
        
        response = self.client.get(reverse('manage_liens_outils'))
        self.assertEqual(response.status_code, 403)

    def test_logged_in(self):
        login = self.client.login(username='testuser2', password='test2passw0rd')
        self.assertTrue(login)
        
        response = self.client.get(reverse('manage_liens_outils'))

        # Check our user is logged in
        self.assertEqual(str(response.context['user']), 'testuser2')
        # Check that we got a response "success"
        self.assertEqual(response.status_code, 200)

        # Check we used correct template
        self.assertTemplateUsed(response, 'manage_liens_outils/liste_liens_outils.html')

    def test_content(self):
        self.client.login(username='testuser2', password='test2passw0rd')
        response = self.client.get(reverse('manage_liens_outils'))
        self.assertEqual(len(response.context['liste_liens']), 10)


@override_settings(AUTHENTICATION_BACKENDS=
                   ('django.contrib.auth.backends.ModelBackend',))
class EditLiensOutilsTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        baker.make(LiensOutils, 10)

    def setUp(self):
        user1 = User.objects.create_user(username='testuser1',
                                         password='test1passw0rd')
        user2 = User.objects.create_user(username='testuser2',
                                         password='test2passw0rd')
        user1.save()
        user2.account_type = 'administrator'
        user2.save()

    def test_redirect_if_not_logged_in(self):
        response = self.client.get(reverse('edit_lien_outils'))
        self.assertRedirects(response, '/?next=/manage_liens_outils/edit_lien/')

    @prevent_request_warnings
    def test_forbidden_if_not_logged_with_admin_user(self):
        login = self.client.login(username='testuser1',
                                  password='test1passw0rd')
        self.assertTrue(login)

        response = self.client.get(reverse('edit_lien_outils'))
        self.assertEqual(response.status_code, 403)

    def test_logged_in(self):
        login = self.client.login(username='testuser2', password='test2passw0rd')
        self.assertTrue(login)

        response = self.client.get(reverse('edit_lien_outils'))

        # Check our user is logged in
        self.assertEqual(str(response.context['user']), 'testuser2')
        # Check that we got a response "success"
        self.assertEqual(response.status_code, 200)

        # Check we used correct template
        self.assertTemplateUsed(response, 'manage_liens_outils/edit_lien_outils.html')

    def test_content(self):
        self.client.login(username='testuser2', password='test2passw0rd')
        response = self.client.get(reverse('edit_lien_outils', kwargs={'lien':1}))
        lien = LiensOutils.objects.get(pk=1).lien
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['manage_lien_form'].initial['lien'], lien)

    def test_submit_change(self):
        self.client.login(username='testuser2', password='test2passw0rd')
        response = self.client.post(reverse('edit_lien_outils', kwargs={'lien':1}),
                                    {'lien': 'http://mon-url.com',
                                     'texte_lien_fr': 'texte fr',
                                     'texte_lien_en': 'texte en',
                                     'description_lien_fr': 'description fr',
                                     'description_lien_en': 'description en'},
                                    follow=True)
        lien = LiensOutils.objects.get(pk=1)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(lien.lien, 'http://mon-url.com')
        self.assertEqual(lien.texte_lien_fr, 'texte fr')
        self.assertEqual(lien.texte_lien_en, 'texte en')
        self.assertEqual(lien.description_lien_fr, 'description fr')
        self.assertEqual(lien.description_lien_en, 'description en')
        
    def test_submit_new(self):
        self.client.login(username='testuser2', password='test2passw0rd')
        response = self.client.post(reverse('edit_lien_outils'),
                                    {'lien': 'http://mon-url.com',
                                     'texte_lien_fr': 'texte fr',
                                     'texte_lien_en': 'texte en',
                                     'description_lien_fr': 'description fr',
                                     'description_lien_en': 'description en'},
                                    follow=True)
        lien = LiensOutils.objects.get(pk=11)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(lien.lien, 'http://mon-url.com')
        self.assertEqual(lien.texte_lien_fr, 'texte fr')
        self.assertEqual(lien.texte_lien_en, 'texte en')
        self.assertEqual(lien.description_lien_fr, 'description fr')
        self.assertEqual(lien.description_lien_en, 'description en')


@override_settings(AUTHENTICATION_BACKENDS=
                   ('django.contrib.auth.backends.ModelBackend',))
class efface_lienTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        baker.make(LiensOutils, 10)

    def setUp(self):
        user1 = User.objects.create_user(username='testuser1',
                                         password='test1passw0rd')
        user2 = User.objects.create_user(username='testuser2',
                                         password='test2passw0rd')
        user1.save()
        user2.account_type = 'administrator'
        user2.save()

    def test_redirect_if_not_logged_in(self):
        response = self.client.get(reverse('efface_lien', kwargs={'lien_pk': 1}))
        self.assertRedirects(response, '/?next=/delete_link/1')
        self.assertTrue(LiensOutils.objects.filter(pk=1).exists())

    @prevent_request_warnings
    def test_redirect_if_not_logged_with_admin_user(self):
        login = self.client.login(username='testuser1',
                                  password='test1passw0rd')
        self.assertTrue(login)

        response = self.client.get(reverse('efface_lien', kwargs={'lien_pk': 1}))
        self.assertRedirects(response, '/?next=/delete_link/1')
        self.assertTrue(LiensOutils.objects.filter(pk=1).exists())

    def test_logged_in(self):
        login = self.client.login(username='testuser2', password='test2passw0rd')
        self.assertTrue(login)

        self.assertTrue(LiensOutils.objects.filter(pk=1).exists())

        response = self.client.get(reverse('efface_lien', kwargs={'lien_pk': 1}),
                                   follow=True)

        # Check our user is logged in
        self.assertEqual(str(response.context['user']), 'testuser2')
        # Check that we got a response "success"
        self.assertEqual(response.status_code, 200)
        self.assertFalse(LiensOutils.objects.filter(pk=1).exists())
